<?php
/**
 * DyMy User Agents
 * Copyright 2012 Dylan Myers
 * Author Dylan Myers
 * http://mybb.dylanspcs.com
 * MyBB 1.6x compatible
 */

// Disallow direct access to this file for security reasons
if(!defined("IN_MYBB"))
{
	die("Direct initialization of this file is not allowed.<br /><br />Please make sure IN_MYBB is defined.");
}

// Neat trick for caching our custom template(s)
// Basically, when we include this from class_plugins.php we can do stuff in init.php, which is before we cache our templates
// So we won't need an extra call to cache it.

if(my_strpos($_SERVER['PHP_SELF'], 'showthread.php'))
{
	global $templatelist;
	if(isset($templatelist))
	{
		$templatelist .= ',';
	}
	$templatelist .= 'DyMyUserAgent_Postbit_OS,DyMyUserAgent_Postbit_Browser';
}

$plugins->add_hook("datahandler_post_insert_post", "dymy_ua_dh_post_insert");
$plugins->add_hook("datahandler_post_insert_thread_post", "dymy_ua_dh_post_insert");
//$plugins->add_hook("datahandler_post_update", "dymy_ua_dh_post_insert");
//$plugins->add_hook("datahandler_post_update_thread", "dymy_ua_dh_post_insert");

//$plugins->add_hook("admin_tools_menu", "useragent_admin_tools_menu");
//$plugins->add_hook("admin_tools_action_handler", "useragent_admin_tools_action_handler");

$plugins->add_hook("postbit", "dymy_ua_postbit");

function dymy_ua_info()
{
	return array(
		"name"					=> "DyMy User Agents",
		"description"			=> "Show User Agent (Browser and OS) icons in posts.",
		"website"				=> "http://mybb.dylanspcs.com",
		"author"				=> "Dylan Myers",
		"authorsite"			=> "mailto:dylanspcs@gmail.com",
		"version"				=> "0.1.7",
		"guid"					=> "600fb849e88f479bdb3c42c57e4070c6",
		"compatibility" 		=> "16*",
	);
}

function dymy_ua_install()
{
	global $db;
	
	$db->write_query("ALTER TABLE ".TABLE_PREFIX."posts ADD `useragent` VARCHAR(255)");
}

function dymy_ua_is_installed()
{
	global $db;
	
	if($db->field_exists("useragent", "posts"))
	{
		return true;
	}
	
	return false;
}

function dymy_ua_uninstall()
{
	global $db;
	
	$db->delete_query("templategroups", "title='DyMy User Agent Templates'");
	$db->write_query("ALTER TABLE ".TABLE_PREFIX."posts DROP `useragent`");
}

function dymy_ua_activate()
{
	global $db;
	
	$q = $db->simple_select("templategroups", "COUNT(*) as count", "title = 'DyMy User Agent'");
	$c = $db->fetch_field($q, "count");
	$db->free_result($q);
	
	if(!($c > 0))
	{
		$ins = array(
			"prefix"		=> "DyMyUserAgent",
			"title"			=> "DyMy User Agent",
		);
		$db->insert_query("templategroups", $ins);
	}
	
	$ins = array(
		"tid"			=> NULL,
		"title"			=> 'DyMyUserAgent_Postbit_OS',
		"template"		=> $db->escape_string(' <img src="images/useragent/os/{$os}.png" alt="{$os}" title="{$os_name}">'),
		"sid"			=> "-2",
		"version"		=> "1601",
		"dateline"		=> time(),
	);
	$db->insert_query("templates", $ins);
	
	$ins = array(
		"tid"			=> NULL,
		"title"			=> 'DyMyUserAgent_Postbit_Browser',
		"template"		=> $db->escape_string(' <img src="images/useragent/browser/{$browser}.png" alt="{$browser}" title="{$browser_name}">'),
		"sid"			=> "-2",
		"version"		=> "1601",
		"dateline"		=> time(),
	);
	$db->insert_query("templates", $ins);
	
	require_once MYBB_ROOT."/inc/adminfunctions_templates.php";
	find_replace_templatesets('postbit',"#".preg_quote('{$post[\'button_delete_pm\']}')."#",'{$post[\'button_delete_pm\']}{$post[\'icon_browser\']}{$post[\'icon_os\']}');
	find_replace_templatesets('postbit_classic',"#".preg_quote('{$post[\'button_delete_pm\']}</td>')."#",'{$post[\'button_delete_pm\']}{$post[\'icon_browser\']}{$post[\'icon_os\']}</td>');
}

function dymy_ua_deactivate()
{
	global $db;
	
	$db->delete_query("templates", "title IN('DyMyUserAgent_Postbit_OS', 'DyMyUserAgent_Postbit_Browser') AND sid='-2'");
	
	require_once MYBB_ROOT."/inc/adminfunctions_templates.php";
	find_replace_templatesets('postbit',"#".preg_quote('{$post[\'icon_browser\']}{$post[\'icon_os\']}')."#",'',0);
	find_replace_templatesets('postbit_classic',"#".preg_quote('{$post[\'icon_browser\']}{$post[\'icon_os\']}</td>')."#",'</td>',0);
}

function dymy_ua_dh_post_insert(&$data)
{
	global $db;

	$useragent = $_SERVER ? $_SERVER['HTTP_USER_AGENT'] : $HTTP_SERVER_VARS['HTTP_USER_AGENT'];
	$data->post_insert_data['useragent'] = $db->escape_string($useragent);
	$data->post_update_data['useragent'] = $db->escape_string($useragent);
}

function dymy_ua_postbit(&$post)
{
	global $templates;
	
	if(isset($post['browser']) && isset($post['system']) && !empty($post['browser']) && !empty($post['system']) && empty($post['useragent']))
	{
		$os = str_ireplace("icon_", "", $post['system']);
		$browser = str_ireplace("icon_", "", $post['browser']);
		$browser = preg_replace("#^linux([a-z])#si", "$1", $browser);
	}
	else
	{
		foreach(explode(";", fetch_remote_file("http://www.useragentstring.com/index.php", array("uas" => $post['useragent'], "getText" => "all"))) as $val)
		{
			$t = explode("=", $val);
			$contents[$t[0]] = $t[1];
		}
		unset($t);
		
		$os = strtolower($contents['os_name']);
		$browser = strtolower($contents['agent_name']);
		/*if(file_exists(MYBB_ROOT."images/useragent/browser/".$browser." ".substr($contents['agent_version'], 0, 1).".png"))
		{
			$browser .= " ".substr($contents['agent_version'], 0, 1);echo "Here!";
		}*/
		if($browser == "internet explorer" && intval($contents['agent_version']) > 6)
		{
			$browser .= " new";
		}
		
		// Special Handling Section!
		// Here we handle special things not picked up by the useragentstring.com API, or special cases from the API
		// Linux Distro is set!
		if(isset($contents['linux_distibution']) && !empty($contents['linux_distibution']) && $contents['linux_distibution'] != "Null") // Bad spelling
		{
			$os = strtolower($contents['linux_distibution']);
		}
		if(isset($contents['linux_distribution']) && !empty($contents['linux_distribution']) && $contents['linux_distribution'] != "Null") // For when spelling fixed.
		{
			$os = strtolower($contents['linux_distribution']);
		}
		// Kindle
		if($browser == "kindle  browser")
		{
			$os = "kindle";
			if(strpos($post['useragent'], "Kindle/3.") != false)
			{
				$contents['os_versionNumber'] = '3';
			}
			if(strpos($post['useragent'], "Kindle/2.") != false)
			{
				$contents['os_versionNumber'] = '2';
			}
		}
		/*if(preg_match("#Silk-Accelerated=#i", $post['useragent']) && !preg_match("#maxthon#i", $post['useragent']))
		{
			$os = "kindle fire";
			$browser = "silk";
		} else*/ if(preg_match("#Silk-Accelerated=#i", $post['useragent']))
		{
			$os = "kindle fire";
		}
		if(strpos($post['useragent'], "Kindle Fire"))
		{
			$os = "kindle fire";
		}
		// Palm WebOS
		if((preg_match("#webOS#i", $post['useragent']) || preg_match("#PalmOS#i", $post['useragent'])) && $os == "unknown")
		{
			$os = "palm";
		}
		// HP TouchPad WebOS
		if(preg_match("#hp-tablet#i", $post['useragent']) && ($os == "unknown" || $os == "linux"))
		{
			$os = "hp";
		}
		// Windows CE as "WindowsCE"
		if(preg_match("#WindowsCE#i", $post['useragent']) && $os == "unknown")
		{
			$os = "windows ce";
		}
		// Playstation 3 & PSP
		if($browser == "playstation 3" || $browser == "playstation portable")
		{
			$os = $browser;
			$usetype = 1;
		}
		// Wii
		if(preg_match("#Nintendo Wii#i", $post['useragent']))
		{
			$os = "nintendo wii";
		}
		if($browser == "safari" && $os == "iphone os")
		{
			$browser .= " iphone";
		}
	}
	
	if(isset($usetype))
	{
		$os_name = make_cool_name($os, 0, "os");
		$browser_name = make_cool_name($browser, 0, "browser");
	}
	else
	{
		$os_name = make_cool_name($os);
		$browser_name = make_cool_name($browser);
	}
	
	// Set version #s!
	if(isset($contents['agent_version']) && !empty($contents['agent_version']))
	{
		$browser_name = str_replace("{v}", $contents['agent_version'], $browser_name);
	}
	else
	{
		$browser_name = str_replace(" {v}", '', $browser_name);
	}
	if((isset($contents['os_versionName']) && !empty($contents['os_versionName'])) && (isset($contents['os_versionNumber']) && !empty($contents['os_versionNumber'])))
	{
		$os_name = str_replace("{v}", $contents['os_versionName']." ".$contents['os_versionNumber'], $os_name);
	}
	else if(isset($contents['os_versionNumber']) && !empty($contents['os_versionNumber']))
	{
		$os_name = str_replace("{v}", $contents['os_versionNumber'], $os_name);
	}
	else if(isset($contents['os_versionName']) && !empty($contents['os_versionName']))
	{
		$os_name = str_replace("{v}", $contents['os_versionName'], $os_name);
	}
	else
	{
		$os_name = str_replace(" {v}", '', $os_name);
	}
	
	if($browser == "playstation 3" || $browser == "playstation portable" || $os == "kindle" || ($os == "kindle fire" && empty($browser)))
	{
		if(!empty($browser))
		{
			$os_name .= " &amp; ".$browser_name;
		}
		eval("\$post['icon_os'] = \"".$templates->get("DyMyUserAgent_Postbit_OS")."\";");
	}
	else
	{
		eval("\$post['icon_browser'] = \"".$templates->get("DyMyUserAgent_Postbit_Browser")."\";");
		eval("\$post['icon_os'] = \"".$templates->get("DyMyUserAgent_Postbit_OS")."\";");
	}
}

// The following function is copied from an older useragent plugn to provide compatibility.
// I'd like to rewrite it at some point with my own version, since this is a sad, sad mess.
// Or perhaps.. I could change my own function just a little & change how its called to combine the functionality.
// 2012-2-10 Made some modifications to split browser and os typing for returning different strings for the consoles.
function make_cool_name($str,$acp=0,$type="both")
{
	$str = str_replace('icon_','',$str);
	
	if($type == "both" || $type == "browser") // 64 Browsers
	{
		switch($str)
		{
			// A (6)
			case 'acoo': return 'Acoo Browser {v}';
			case 'acoo browser': return 'Acoo Browser {v}';
			case 'america online browser': return 'AOL Browser {v}';
			case 'amigavoyager': return 'Amiga Voyager {v}';
			case 'aol': return 'AOL Browser {v}';
			case 'arora': return 'Arora {v}';
			case 'avant': return 'Avant Browser {v}';
			case 'avant browser': return 'Avant Browser {v}';
			case 'android webkit browser': return 'Android Browser {v}';
			// B (3)
			case 'beonex': return 'Beonex Communicator {v}';
			case 'blackberry': return 'BlackBerry Browser {v}';
			case 'bolt': return 'Bolt {v}';
			// C (5)
			case 'cometbird': return 'CometBird {v}';
			case 'chrome': return 'Google Chrome {v}';
			case 'chromeplus': return 'ChromePlus {v}';
			case 'comodo_dragon': return 'Comodo Dragon {v}';
			case 'camino': return 'Camino {v}';
			// D (1)
			case 'dillo': return 'Dillo {v}';
			// E (2)
			case 'epiphany': return 'Epiphany {v}';
			case 'elinks': return 'Elinks {v}';
			// F (6)
			case 'flock': return 'Flock {v}';
			case 'firefox': return 'Firefox {v}';
			case 'fennec': return 'Firefox for Mobile {v}';
			case 'firebird': return 'Firebird {v}';
			case 'fireweb navigator': return 'Fireweb Navigator {v}';
			case 'fluid': return 'Fluid {v}';
			// G (2)
			case 'galeon': return 'Galeon {v}';
			case 'greenbrowser': return 'GreenBrowser {v}';
			// H
			// I (6)
			case 'iceweasel': return 'Iceweasel {v}';
			case 'icecat': return 'IceCat {v}';
			case 'iceape': return 'Iceape {v}';
			case 'ie': return 'Internet Explorer 6 or Below';
			case 'ie7': return 'Internet Explorer 7';
			case 'ie8': return 'Internet Explorer 8';
			case 'ie mobile': return 'Internet Explorer Mobile {v}';
			case 'internet explorer': return 'Internet Explorer {v}';
			case 'internet explorer new': return 'Internet Explorer {v}';
			case 'iris': return 'Iris Mobile Browser {v}';
			// J
			// K (4)
			case 'konqueror': return 'Konqueror {v}';
			case 'k-meleon': return 'K-Meleon {v}';
			case 'kapiko': return 'Kapiko {v}';
			case 'kazehakase': return 'Kazehakase {v}';
			case 'kindle  browser': return 'Kindle Browser';
			// L (4)
			case 'lunascape': return 'Lunascape {v}';
			case 'lobo': return 'Lobo {v}';
			case 'links': return 'Links {v}';
			case 'lynx': return 'Lynx {v}';
			// M (4)
			case 'mozilla': return 'Mozilla {v}';
			case 'maxthon': return 'Maxthon {v}';
			case 'midori': return 'Midori {v}';
			case 'minimo': return 'Minimo {v}';
			// N (3)
			case 'netscape': return 'Netscape {v}';
			case 'netsurf': return 'NetSurf {v}';
			case 'netfront': return 'NetFront {v}';
			// O (4)
			case 'opera': return 'Opera {v}';
			case 'operamini': return 'Opera Mini {v}';
			case 'opera mini': return 'Opera Mini {v}';
			case 'opera mobile': return 'Opera Mobile {v}';
			case 'omniweb': return 'OmniWeb {v}';
			// P (4)
			case 'playstation_3': return 'PS3 Browser {v}';
			case 'playstation 3': return 'PS3 Browser {v}';
			case 'playstation_portable': return 'PSP Browser {v}';
			case 'playstation portable': return 'PSP Browser {v}';
			case 'phoenix': return 'Phoenix {v}';
			case 'palemoon': return 'Pale Moon {v}';
			// Q (1)
			case 'qtweb internet browser': return 'QtWeb Internet Browser {v}';
			// R (2)
			case 'rekonq': return 'Rekonq {v}';
			case 'rockmelt': return 'RockMelt {v}';
			// S (9)
			case 'seamonkey': return 'SeaMonkey {v}';
			case 'slimbrowser': return 'SlimBrowser {v}';
			case 'safari_iphone': return 'Safari for iPhone {v}';
			case 'safari iphone': return 'Safari for iPhone {v}';
			case 'safari': return 'Safari {v}';
			case 'sunrise': return 'Sunrise {v}';
			case 'stainless': return 'Stainless {v}';
			case 'shiira': return 'Shiira {v}';
			case 'skyfire': return 'Skyfire {v}';
			case 'silk': return 'Amazon Silk {v}';
			// T (2)
			case 'thunderbird': return 'ThunderBird {v}';
			case 'teashark': return 'TeaShark {v}';
			// U (1)
			case 'uzbl': return 'Uzbl {v}';
			// V
			// W (3)
			case 'w3m': return 'W3M {v}';
			case 'www': return 'WorldWideWeb {v}';
			case 'worldwideweb': return 'WorldWideWeb {v}';
			case 'wyzo': return 'Wyzo {v}';
			// X
			// Y
			// Z
			
			// None results
			case 'noneb': return 'no Browser';
			case 'unknown_b': return $str;
			default: return $str;
		}
	}
	
	if($type == "both" || $type == "os") // 53 OSes
	{
		switch($str)
		{
			// "Others" (4)
			case 'amiga': return 'Amiga {v}';
			case 'beos': return 'BeOS {v}';
			case 'haiku': return 'Haiku {v}';
			case 'symbos': return 'SymbOS {v}';
			
			// Smartphones & Tablets (6)
			case 'symbian': return 'Symbian {v}';
			case 'symbianos': return 'Symbian {v}';
			case 'symbian os': return 'Symbian {v}';
			case 'blackberryos': return 'BlackBerry OS {v}';
			case 'maemo': return 'Maemo {v}';
			case 'kindle': return 'Kindle {v}';
			case 'kindle fire': return 'Kindle Fire (Android) {v}';
			case 'palm': return 'Palm WebOS {v}';
			case 'hp': return 'HP TouchPad WebOS {v}';
			
			// Apple OSes (3)
			case 'macos': return 'MacOS {v}';
			case 'macintosh': return 'MacOS {v}';
			case 'os x': return 'Mac OS X {v}';
			case 'iphone': return 'an iOS Device {v}';
			case 'iphone os': return 'an iOS Device {v}';
			
			// Consoles (3)
			case 'playstation_3': return 'a Playstation 3 {v}';
			case 'playstation 3': return 'a Playstation 3 {v}';
			case 'playstation_portable': return 'a PSP {v}';
			case 'playstation portable': return 'a PSP {v}';
			case 'nintendo_wii': return 'a Nintendo Wii {v}';
			case 'nintendo wii': return 'a Nintendo Wii {v}';
			
			// Linux Distributions (18)
			case 'debian': return 'Debian {v}';
			case 'fedora': return 'Fedora {v}';
			case 'gentoo': return 'Gentoo {v}';
			case 'kateos': return 'KateOS {v}';
			case 'knoppix': return 'Knoppix {v}';
			case 'kubuntu': return 'Kubuntu {v}';
			case 'mandriva': return 'Mandriva {v}';
			case 'redhat': return 'Red Hat {v}';
			case 'red hat': return 'Red Hat {v}';
			case 'centos': return 'Cent OS {v}';
			case 'slackware': return 'Slackware {v}';
			case 'suse': return 'SuSE {v}';
			case 'ubuntu': return 'Ubuntu {v}';
			case 'linux mint': return 'Linux Mint {v}';
			case 'arch': return 'Archlinux {v}';
			case 'android': return 'Android {v}';
			case 'chrome os': return 'Chrome OS {v}';
			case 'kanotix': return 'Kanotix {v}';
			case 'linux': return 'Linux {v}';
			
			// BSD/UNIX (8)
			case 'freebsd': return 'FreeBSD {v}';
			case 'netbsd': return 'NetBSD {v}';
			case 'openbsd': return 'OpenBSD {v}';
			case 'solaris': return 'Solaris {v}';
			case 'sunos': return 'SunOS {v}';
			case 'dragonfly': return 'DragonFly BSD {v}';
			case 'irix': return 'Irix {v}';
			case 'unix': return 'UNIX {v}';
			
			// Windows (11)
			case 'windows ce': return 'Windows CE {v}';
			case 'windows_98_nt_2000': return 'Windows 9x/NT/2000 {v}';
			case 'windows nt': return 'Windows NT {v}';
			case 'windows_me': return 'Windows ME {v}';
			case 'windows me': return 'Windows ME {v}';
			case 'windows 3.1': return 'Windows 3.1 {v}';
			case 'windows 95': return 'Windows 95 {v}';
			case 'windows 98': return 'Windows 98 {v}';
			case 'windows 2000': return 'Windows 2000 {v}';
			case 'windows_xp_2003': return 'Windows XP/2003 {v}';
			case 'windows xp': return 'Windows XP {v}';
			case 'windows server 2003': return 'Windows 2003 {v}';
			case 'windows_vista': return 'Windows Vista {v}';
			case 'windows vista': return 'Windows Vista {v}';
			case 'windows_seven': return 'Windows Seven {v}';
			case 'windows 7': return 'Windows Seven {v}';
			case 'unknown_b': return $str;
			default: return $str;
		}
	}
}

?>
